// *********************************************************************
// *                       E08 - TEMPERATURE                           *
// *                          TEMPERATURE                              *
// *********************************************************************
// Temperature measurement.
//
// Intended for:
//    HW: - IQRF transceiver modules
//          (TR-21A HW v1.03 - black, TR-31B or higher)
//        - CK-USB-02 development kit
//    OS: v2.xx
//
// Description:
//    Temperature is measured and sent via SPI in 5 s period.
//
//
// File:    E07-TEMPERATURE.c
// Version: v1.03                                   Revision: 26/11/2009
//
// Revision history:
//     v1.03: 26/11/2009  Just comments improved.
//     v1.02: 24/11/2008  Verbose comments
//     v1.01: 22/03/2008  First release
// *********************************************************************

#include "includes/template-basic.h"

// *********************************************************************

void APPLICATION()
{
    uns8    pomi;
    uns16   temperature;

    enableSPI();

    while (1)
    {
        clrwdt();
        getTemperature();                     // Temperature measurement
	    temperature.high8 = ADRESH&0x03;  // 10 b result is stored
	    temperature.low8  = ADRESL;       //   in ADRESH and ADRESL


                // Vout = Tc * Ta + Vo       Tc = 10 mV/C   Vo = 500 mV
                //                              see MCP9700 datasheet

                // Vout = Vref * ADRES/1024   Vref = 3 V
                //                              see PIC16F88/6 datasheet

	 	// Ta = 75/256 * ADRES - 50   [C]

		temperature *= 75;
		temperature >>= 8;
		temperature -= 50;

        bufferCOM[0] = temperature.low8/10;
        bufferCOM[0] += '0';
        bufferCOM[1] = temperature.low8%10;
        bufferCOM[1] += '0';                  // Display in ttC format
        bufferCOM[2] = 'C';
        startSPI(3);                          // Send Ta via SPI
        waitDelay(250);
        clrwdt();
        waitDelay(250);                       // Wait 5 s
    }
}

// *********************************************************************

